@@ -38,6 +38,7 @@
 #include "gerror.h"
 
 #include <gjs/compat.h>
+#include <gjs/jsapi-private.h>
 
 #include <util/log.h>
 #include <util/misc.h>
@@ -148,6 +149,7 @@ resolve_namespace_object(JSContext  *context,
               "Defined namespace '%s' %p in GIRepository %p", ns_name, gi_namespace, repo_obj);
 
     ret = JS_TRUE;
+    gjs_schedule_gc_if_needed(context);
 
  out:
     if (gi_namespace)
